@@ -195,8 +195,9 @@ prepare_delete_bounds (HTMLEngine *e, GList **from_list, GList **to_list,
 
 	common_parent = get_common_parent (begin.object, end.object);
 	ret_level     = html_object_get_parent_level (common_parent);
-	/* printf ("common parent level: %d\n", ret_level); */
-
+#ifdef OP_DEBUG
+	printf ("common parent level: %d\n", ret_level);
+#endif
 	*from_list = point_get_parent_list (&begin, get_parent_depth (begin.object, common_parent), TRUE);
 	*to_list   = point_get_parent_list (&end,   get_parent_depth (end.object, common_parent),   TRUE);
 
@@ -336,22 +337,22 @@ static void
 split_and_add_empty_texts (HTMLEngine *e, gint level, GList **left, GList **right)
 {
 #ifdef OP_DEBUG
-	printf ("-- SPLIT begin\n");
+	printf ("-- SPLIT begin\ne->clue:\n");
 	gtk_html_debug_dump_tree_simple (e->clue, 0);
 	printf ("-- SPLIT middle\n");
 #endif
 	html_object_split (e->cursor->object, e, *right ? HTML_OBJECT ((*right)->data) : NULL,
 			   e->cursor->offset, level, left, right);
 #ifdef OP_DEBUG
-	printf ("-- SPLIT middle\n");
+	printf ("-- SPLIT middle\ne->clue:\n");
 	gtk_html_debug_dump_tree_simple (e->clue, 0);
 	printf ("-- SPLIT finish\n");
 	if (*left && (*left)->data) {
-		printf ("left\n");
+		printf ("left:\n");
 		gtk_html_debug_dump_tree_simple (HTML_OBJECT ((*left)->data), 0);
 	}
 	if (*right && (*right)->data) {
-		printf ("right\n");
+		printf ("right:\n");
 		gtk_html_debug_dump_tree_simple (HTML_OBJECT ((*right)->data), 0);
 	}
 	printf ("-- SPLIT end\n");
@@ -423,15 +424,18 @@ delete_setup_undo (HTMLEngine *e, HTMLObject *buffer, guint len, guint position_
 
 	undo = g_new (DeleteUndo, 1);
 
-	/* printf ("cursor level: %d undo level: %d\n", html_object_get_parent_level (e->cursor->object), level); */
+#ifdef OP_DEBUG
+	printf ("cursor level: %d undo level: %d\n", html_object_get_parent_level (e->cursor->object), level);
+#endif
 	html_undo_data_init (HTML_UNDO_DATA (undo));
 	undo->data.destroy = delete_undo_destroy;
 	undo->buffer       = buffer;
 	undo->buffer_len   = len;
 	undo->level        = level;
 
-	/* printf ("delete undo len %d\n", len); */
-
+#ifdef OP_DEBUG
+	printf ("delete undo len %d\n", len);
+#endif
 	html_undo_add_action (e->undo,
 			      html_undo_action_new ("Delete object", delete_undo_action,
 						    HTML_UNDO_DATA (undo), html_cursor_get_position (e->cursor),
@@ -1076,6 +1080,7 @@ insert_object_for_undo (HTMLEngine *e, HTMLObject *obj, guint len, guint positio
 	html_cursor_jump_to_position_no_spell (e->cursor, e, position_after + (delete_paragraph_before ? 1 : 0));
 	insert_setup_undo (e, len, position_before + (delete_paragraph_before ? 1 : 0),
 			   dir, delete_paragraph_before, delete_paragraph_after);
+	g_signal_emit_by_name (e->widget, "object_inserted", position_before, len);
 }
 
 static void
@@ -1097,7 +1102,7 @@ html_engine_paste_object (HTMLEngine *e, HTMLObject *o, guint len)
 {
 	html_undo_level_begin (e->undo, "Paste", "Paste");
 	html_engine_delete (e);
-	html_engine_insert_object (e, o, len, html_object_get_insert_level (o));
+	html_engine_insert_object (e, o, len, html_engine_get_insert_level_for_object (e, o));
 	html_undo_level_end (e->undo);
 }
 
@@ -1190,12 +1195,21 @@ insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo)
 		html_undo_level_end (e->undo);
 
 	html_engine_thaw (e);
+	g_signal_emit_by_name (e->widget, "object_inserted", 0, 0);
 }
 
 void
 html_engine_insert_empty_paragraph (HTMLEngine *e)
 {
+	HTMLClueFlow *cf;
+
+	html_engine_freeze (e);
 	insert_empty_paragraph (e, HTML_UNDO_UNDO, TRUE);
+	cf = html_object_get_flow (e->cursor->object);
+	if (cf) {
+		cf->dir = html_text_direction_pango_to_html (gdk_keymap_get_direction (gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (e->widget)))));
+	}
+	html_engine_thaw (e);
 }
 
 static char *picto_chars = "DO)(|/PQ\0:-\0:\0:-\0:\0:;=-\0:;\0:-~\0:\0:\0:-\0:\0:-\0:\0:-\0:\0:-\0:\0";
@@ -1272,13 +1286,13 @@ use_pictograms (HTMLEngine *e)
 }
 
 void
-html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
+html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text, gint len, PangoAttrList *attrs)
 {
 	gchar *nl;
 	gint alen;
+	gsize bytes;
 
-	if (len == -1)
-		len = g_utf8_strlen (text, -1);
+	bytes = html_text_sanitize (&text, &len);
 	if (!len)
 		return;
 
@@ -1287,7 +1301,7 @@ html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text,
 	gtk_html_editor_event_command (e->widget, GTK_HTML_COMMAND_INSERT_PARAGRAPH, TRUE);
 
 	do {
-		nl   = g_utf8_strchr (text, -1, '\n');
+		nl   = memchr (text, '\n', bytes);
 		alen = nl ? g_utf8_pointer_to_offset (text, nl) : len;
 		if (alen) {
 			HTMLObject *o;
@@ -1318,7 +1332,8 @@ html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text,
 		}
 		if (nl) {
 			html_engine_insert_empty_paragraph (e);
-			len -= g_utf8_pointer_to_offset (text, nl) + 1;
+			len -= alen + 1;
+			bytes -= (nl - text) + 1;
 			text = nl + 1;
 		}
 	} while (nl);
@@ -1326,7 +1341,7 @@ html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text,
 }
 
 void
-html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
+html_engine_insert_text (HTMLEngine *e, const gchar *text, gint len)
 {
 	html_engine_insert_text_with_extra_attributes (e, text, len, NULL);
 }
@@ -1700,6 +1715,37 @@ delete_upto (HTMLEngine *e, HTMLCursor **start, HTMLCursor **end, HTMLObject *ob
 	(*end)->position -= position - e->cursor->position;
 }
 
+static gboolean
+check_for_simple_containers (HTMLObject *child, HTMLObject *parent)
+{
+	while (child && child != parent) {
+		if (html_object_is_container (child)) {
+			switch (child->klass->type) {
+			case HTML_TYPE_CLUEFLOW:
+			case HTML_TYPE_CLUEV:
+			case HTML_TYPE_TABLECELL:
+				break;
+			default:
+				return FALSE;
+			}
+		}
+		child = child->parent;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+check_for_simple_delete (HTMLObject *start, HTMLObject *end)
+{
+	HTMLObject *common_parent = get_common_parent (start, end);
+
+	if (common_parent && check_for_simple_containers (start->parent, common_parent) && check_for_simple_containers (end->parent, common_parent))
+		return TRUE;
+
+	return FALSE;
+}
+
 void
 html_engine_delete (HTMLEngine *e)
 {
@@ -1709,9 +1755,11 @@ html_engine_delete (HTMLEngine *e)
 		HTMLCursor *start = html_cursor_dup (e->mark->position < e->cursor->position ? e->mark : e->cursor);
 		HTMLCursor *end = html_cursor_dup (e->mark->position < e->cursor->position ? e->cursor : e->mark);
 		gint start_position = start->position;
+		gint end_position = end->position;
+
 
 		while (start->position < end->position) {
-			if (start->object->parent->parent == end->object->parent->parent) {
+			if (check_for_simple_delete (start->object, end->object)) {
 				if (e->mark)
 					html_cursor_destroy (e->mark);
 				html_cursor_destroy (e->cursor);
@@ -1754,6 +1802,11 @@ html_engine_delete (HTMLEngine *e)
 		if (end)
 			html_cursor_destroy (end);
 		html_cursor_jump_to_position (e->cursor, e, start_position);
+
+		if (end_position - start_position > 0) {
+			int len = end_position - start_position;
+			g_signal_emit_by_name (e->widget, "object_deleted", start_position, len);
+		}
 	}
 	html_undo_level_end (e->undo);
 }
