@@ -322,9 +322,11 @@ get_bg_color (HTMLObject *o,
 	if (o->parent)
 		return html_object_get_bg_color (o->parent, p);	
 
-	if (p->widget && GTK_IS_HTML (p->widget))
-		return &((html_colorset_get_color (GTK_HTML (p->widget)->engine->settings->color_set,
-						   HTMLBgColor))->color);
+	if (p->widget && GTK_IS_HTML (p->widget)) {
+		HTMLEngine *e = html_object_engine (o, GTK_HTML (p->widget)->engine);
+		return &((html_colorset_get_color (e->settings->color_set, HTMLBgColor))->color);
+	}
+
 	return NULL;
 }
 
@@ -621,19 +623,37 @@ html_object_real_cursor_forward (HTMLObject *self, HTMLCursor *cursor)
 		return FALSE;
 }
 
+static gboolean
+html_cursor_allow_zero_offset (HTMLCursor *cursor, HTMLObject *o)
+{
+	if (cursor->offset == 1) {
+		HTMLObject *prev;
+
+		prev = html_object_prev_not_slave (o);
+		if (!prev || HTML_IS_CLUEALIGNED (prev))
+			return TRUE;
+		else {
+			while (prev && !html_object_accepts_cursor (prev))
+				prev = html_object_prev_not_slave (prev);
+
+			if (!prev)
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static gboolean
 html_object_real_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 {
-	HTMLObject *prev;
-
 	g_assert (self);
 	g_assert (cursor->object == self);
 
 	if (html_object_is_container (self))
 		return FALSE;
 
-	if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-							  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
+	if (cursor->offset > 1 || html_cursor_allow_zero_offset (cursor, self)) {
 		cursor->offset --;
 		cursor->position --;
 		return TRUE;
@@ -664,10 +684,7 @@ html_object_real_cursor_right (HTMLObject *self, HTMLPainter *painter, HTMLCurso
 			return TRUE;
 		}
 	} else {
-		HTMLObject *prev;
-
-		if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-								  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
+		if (cursor->offset > 1 || html_cursor_allow_zero_offset (cursor, self)) {
 			cursor->offset --;
 			cursor->position --;
 			return TRUE;
@@ -689,9 +706,7 @@ html_object_real_cursor_left (HTMLObject *self, HTMLPainter *painter, HTMLCursor
 		return FALSE;
 
 	if (dir != HTML_DIRECTION_RTL) {
-		HTMLObject *prev;
-		if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-								  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
+		if (cursor->offset > 1 || html_cursor_allow_zero_offset (cursor, self)) {
 			cursor->offset --;
 			cursor->position --;
 			return TRUE;
