@@ -618,7 +618,7 @@ html_painter_draw_entries (HTMLPainter *painter, gint x, gint y,
 			
 			tab = memchr (c_text + 1, (unsigned char) '\t', bytes - 1);
 		} else {
-			x += (* HP_CLASS (painter)->draw_glyphs) (painter, x, y, item, gl->data, NULL, NULL);
+			x += html_painter_pango_to_engine (painter, (* HP_CLASS (painter)->draw_glyphs) (painter, x, y, item, gl->data, NULL, NULL));
 
 			if (line_offset != -1)
 				line_offset += g_utf8_pointer_to_offset (c_text, next);
@@ -820,11 +820,9 @@ html_painter_get_pixel_size (HTMLPainter *painter)
 }
 
 gint
-html_painter_draw_spell_error (HTMLPainter *painter,
-			       gint x, gint y,
-			       HTMLTextPangoInfo *pi, GList *glyphs)
+html_painter_draw_spell_error (HTMLPainter *painter, int x, int y, int width)
 {
-	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, pi, glyphs);
+	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, width);
 }
 
 HTMLFont *
@@ -858,7 +856,8 @@ html_painter_alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboole
 			      text_width (painter, desc, "\t", 1),
 			      text_width (painter, desc, "e", 1),
 			      text_width (painter, desc, HTML_BLOCK_INDENT, strlen (HTML_BLOCK_INDENT)),
-			      text_width (painter, desc, HTML_BLOCK_CITE, strlen (HTML_BLOCK_CITE)));
+			      text_width (painter, desc, HTML_BLOCK_CITE_LTR, strlen (HTML_BLOCK_CITE_LTR)),
+			      text_width (painter, desc, HTML_BLOCK_CITE_RTL, strlen (HTML_BLOCK_CITE_RTL)));
 }
 
 void
@@ -906,9 +905,10 @@ html_painter_get_block_indent_width (HTMLPainter *painter, GtkHTMLFontStyle styl
 }
 
 guint
-html_painter_get_block_cite_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
+html_painter_get_block_cite_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face, HTMLDirection dir)
 {
-	return html_font_manager_get_font (&painter->font_manager, face, style)->cite_width;
+	HTMLFont *font = html_font_manager_get_font (&painter->font_manager, face, style);
+	return dir == HTML_DIRECTION_RTL ? font->cite_width_rtl : font->cite_width_ltr ;
 }
 
 guint
